#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <limits.h>
#include "mbs.h"
#include "sx.c"

unsigned short *fetch(const char *s, size_t l);

/* ERRCODE POS COUNT [[CODE POS LEN S(n)] ...] */

const char *opestr[] = { ".", "?", "+", "*", "(", ")", "|", };
const short opeval[] = { DOT, QST, PLS, AST, GRO, GRC, LOR, };
const int nops = (sizeof opestr) / (sizeof (opestr[0]));

#define ADD(p, cnt, code, dpos, dlen, snum) { \
  if (p) { \
    (p)[(cnt) * 4 + 0] = code; \
    (p)[(cnt) * 4 + 1] = dpos; \
    (p)[(cnt) * 4 + 2] = dlen; \
    (p)[(cnt) * 4 + 3] = snum; \
  } \
  (cnt) += 1; \
}

static unsigned short dofetch(
 unsigned short *err, unsigned short *pos, 
 unsigned short *tp, const char *s, size_t l);

unsigned short *fetch(const char *s, size_t l) {
  unsigned short *p;
  unsigned short ec, ep, n, ex;
  int i;
  if (l <= (USHRT_MAX - 2)) {
    n = dofetch(&ec, &ep, NULL, s, l);
    ex = (n ? ep + (ep * ep) : 0); 
    /* space for (lookup + followpos) */
    if (p = (unsigned short *)malloc(
     sizeof (unsigned short) * (3 + n * 4 + ex))) {
      memset(p, 0, sizeof (unsigned short) * (3 + n * 4 + ep));
      p[0] = ec;
      p[1] = ep;
      p[2] = n;
      if (n) {
        dofetch(&ec, &ep, &(p[3]), s, l);
        for (i = 0; i < n; i++) {
          if (SN(p, i)) {
	    TN(p, SN(p, i)) = i;
	  }
        }
	memset(FPBUFF(p), 0xff, ep * ep * sizeof (unsigned short));
      }
    }
    return p;
  } else {
    return NULL;
  }
  return NULL;
}

#define MBSCMP(s, l, k) \
 (((l) == strlen(k)) ? (memcmp(s, k, l) == 0) : 0)

static int getopeval(const char *s, int m, int b) {
  int i;
  for (i = 0; i < nops; i++) {
    if (MBSCMP(&(s[m]), b, opestr[i])) {
      return opeval[i];
    }
  }
  return 0;
}

static const char *getopesym(int k) {
  int i;
  for (i = 0; i < nops; i++) {
    if (opeval[i] == k) {
      return opestr[i];
    }
  }
  return "NULL";
}

static unsigned short dofetch(unsigned short *err, unsigned short *pos,
 unsigned short *tp, const char *s, size_t l) {
  size_t (*clen)(const char *p, size_t i, size_t l);
  size_t i, b;
  short snum = 1; /* state-number */
  unsigned short cnt = 0; /* output data size */
  int stt = 0, brk = 0, nst = 0; /* state, break-switch paren-nest */
  int cs, cl, ce; /* class start, length escape */
  int es, el, ec; /* escape start, length, column */
  int ps, pl, pf; /* posix-class start, length, flag */
  int c;
  *err = *pos = 0;
  clen = sjisclen;
  for (i = 0; (b = clen(s, i, l)) && !brk; i += b) {
    switch (stt) {
    case 0:
      if (s[i] == '[') {
	cs = i + 1;
	cl = 0;
	ce = 0;
        stt = 1;
      } else if (s[i] == '\\') {
        es = i;
        stt = 2;
      } else {
        if (s[i] == '(') {
	  nst += 1;
	} else if (s[i] == ')') {
	  nst -= 1;
	}
	if (c = getopeval(s, i, b)) {
	  ADD(tp, cnt, c, i, b, 0);
	} else {
	  ADD(tp, cnt, CHR, i, b, snum);
	  snum += 1;
	}
      }
      break;
    case 1: /* class */
      if (ce && s[i] == 'x') {
        ce = 0;
	es = i + 1;
	el = 0;
	ec = 2;
	cl += b;
        stt = 4;
      } else if (ce && s[i] == 'X') {
        ce = 0;
	es = i + 1;
	el = 0;
	ec = 8;
	cl += b;
        stt = 4;
      } else if (ce) {
        cl += b;
        ce = 0;
      } else if ((s[i] == ']') && (cl != 0) 
       && !((cl == 1) && (s[cs] == '^'))) {
        ADD(tp, cnt, SET, cs, cl, snum);
	snum += 1;
        stt = 0;
      } else if (s[i] == '[') {
        pl = 0;
	cl += b;
	stt = 5;
      } else if (s[i] == '\\') {
        ce = 1;
	cl += b;
      } else {
        cl += b;
      }
      break;
    case 2: /* escape */
      if (s[i] == 'x') {
	el = 2;
	ec = 2 + 2;
        stt = 3;
      } else if (s[i] == 'X') {
	el = 2;
	ec = 2 + 8;
        stt = 3;
      } else {
        ADD(tp, cnt, CHR, es, 1 + b, snum);
	snum += 1;
	stt = 0;
      }
      break;
    case 3: /* hex */
      if (isxdigit(s[i])) {
        el++;
	if (el == ec) {
	  ADD(tp, cnt, CHR, es, el, snum);
	  snum += 1;
	  stt = 0;
	}
      } else {
	 brk = 1;
      }
      break;
    case 4: /* hex(class) */
      if (isxdigit(s[i])) {
        el++;
	cl += b;
	if (el == ec) {
	  stt = 1;
	}
      } else { /* error */
        brk = 1;
      }
      break;
    case 5: /* [:A:] */
      if (!pl) {
        if (s[i] == ':') {
          ps = i;
	  pf = 0;
	  pl += 1;
	} else if (s[i] == ']') {
          ADD(tp, cnt, SET, cs, cl, snum);
	  snum += 1;
          stt = 0;
	} else {
	  stt = 1;
	}
      } else if (pf) {
        if (s[i] == ']') {
#	  if 0
	  {
	    putchar('[');
	    fwrite(&(s[ps]), 1, pl, stdout);
	    putchar(']');
	    putchar('\n');
	  }
#         endif
	  stt = 1; /* name ::= s[ps..(ps + pl)] */
	} else {
	  brk = 1;
	}
      } else {
        if (s[i] == ':') {
	  pf = 1;
	}
	pl += 1;
      }
      cl += b;
      break;
    }
  }
  switch (stt) {
  case 0: 
    if (nst == 0) { /* success */
      *err = 0;
      *pos = snum + 1; /* leaf count + accept */
    } else { /* () not matched */
      *err = 101;
      *pos = 0;
      return 0;
    }
    break;
  case 1: /* [] not matched */
    *err = 101;
    *pos = cs;
    return 0;
  case 2: /* \? */
    *err = 102;
    *pos = i;
    return 0;
  case 3: /* \xNN */
    *err = 103;
    *pos = es;
    return 0;
  case 4: /* [\xNN] */
    *err = 104;
    *pos = es;
    return 0;
  case 5: /* [[:xxx:]] not matched */
    *err = 105;
    *pos = ps;
    return 0;
  }
  return cnt;
}

static void labelout(const char *buff, unsigned short *p, int n) {
  const char *s = &(buff[TS(p, n)]);
  char *lc, *rc;
  size_t l = TL(p, n);
  switch (TV(p, n)) {
  case CHR: lc = "'"; rc = "'"; break;
  case SET: lc = "["; rc = "]"; break;
  default:  lc = "" ; rc = "" ; break;
  }
  printf("%s", lc);
  fwrite(s, 1, l, stdout);
  printf("%s", rc);
}

int main() {
  char buff[255];
  size_t n, i;
  unsigned short *p; 
  int c;
# if 0
  yyTraceFILE = stdout;
  yyTracePrompt = "+";
# endif
  for (;;) {
    //putchar('>');
    gets(buff);
    n = strlen(buff);
    if ((n = strlen(buff)) == 0) break;
    if (feof(stdin)) break;
    if (p = fetch(buff, n)) {
      void *pstack = alamoAlloc(malloc);
#if 0
      printf("STAT=%d,POS=%d,LEN=%d\n", p[0], NSTATE(p), NTOKEN(p));
      for (i = 0; i < NTOKEN(p); i++) {
        printf("%d:", SN(p, i));
        fwrite(&(buff[TS(p, i)]), 1, TL(p, i), stdout);
        printf(",");
      }
      printf("\n");
#endif
      for (i = 0; i < NTOKEN(p); i++) {
        alamo(pstack, TV(p, i), i, p);
      }
      alamo(pstack, EOE, 0, p);
      alamo(pstack, 0, 0, p);
      alamoFree(pstack, free);
#if 0
      { 
        printf("{TN:");
        for (i = 0; i < NSTATE(p); i++) {
	  printf("%d,", TN(p, i));
        }
        printf("}\n");
      }
      { int y, x;
        for (y = 0; y < ACCNUM(p); y++) {
          for (x = 0; x < NSTATE(p); x++) {
	    if (FOLLOWPOS(p, y, x) != 0xffff) {
	      printf("S%d->S%d {", y, x);
	      labelout(buff, p, FOLLOWPOS(p, y, x));
	      printf("}\n");
	    }
          }
        }
	for (x = 0; x < NSTATE(p); x++) {
	  if (FOLLOWPOS(p, ACCNUM(p), x) != 0xffff) {
	    printf("ACC:%d\n", x);
	  }
	}
      }
#endif


      printf("digraph \"G\" {\n");
      printf("graph [rankdir=\"LR\" label=\"%s\"];\n", buff);
      printf("node [fontname=\"MSGOTHIC.TTC\", shape=\"circle\"];\n");
      printf("edge [fontname=\"MSGOTHIC.TTC\"];\n");
      { int y, x;
        for (y = 0; y < ACCNUM(p); y++) {
          for (x = 0; x < NSTATE(p); x++) {
	    if (FOLLOWPOS(p, y, x) != 0xffff) {
	      printf("S%d->S%d [label=\"", y, x);
	      labelout(buff, p, FOLLOWPOS(p, y, x));
	      printf("\"]\n");
	    }
          }
        }
        for (x = 0; x < NSTATE(p); x++) {
	  if (FOLLOWPOS(p, ACCNUM(p), x) != 0xffff) {
	    printf("S%d [peripheries=2];\n", x);
	  }
	}
      } 
      printf("}\n");


      free(p);
    }
    putchar('\n');
  }
}
